package cast

import (
	"context"
	"dubbo.apache.org/dubbo-go/v3/common/logger"
	"errors"
	"fonchain-fiee/api/accountFiee"
	"fonchain-fiee/api/bundle"
	"fonchain-fiee/api/cast"
	"fonchain-fiee/cmd/config"
	"fonchain-fiee/pkg/e"
	modelCast "fonchain-fiee/pkg/model/cast"
	"fonchain-fiee/pkg/service"
	"github.com/gin-gonic/gin"
	"strconv"
)

func MediaUserList(ctx *gin.Context) {
	var req *cast.MediaUserListReq
	var err error
	if err = ctx.ShouldBind(&req); err != nil {
		service.Error(ctx, err)
		return
	}
	if err = req.Validate(); err != nil {
		service.Error(ctx, err)
		return
	}
	resp, err := service.CastProvider.MediaUserList(ctx, req)
	if err != nil {
		service.Error(ctx, err)
		return
	}
	service.Success(ctx, resp)
	return
}

func UnbindManager(ctx *gin.Context) {
	var req *cast.UnbindManagerReq
	var resp *cast.UnbindManagerResp
	var err error
	if err = ctx.ShouldBind(&req); err != nil {
		service.Error(ctx, err)
		return
	}
	if err = req.Validate(); err != nil {
		service.Error(ctx, err)
		return
	}
	resp, err = service.CastProvider.UnbindManager(ctx, req)
	if err != nil {
		service.Error(ctx, err)
		return
	}
	logger.Error("UnbindManager resp" + resp.OldBindArtistUuid)
	if resp.OldBindArtistUuid != "" {
		userID, _ := strconv.ParseInt(resp.OldBindArtistUuid, 10, 64)
		_, err = service.BundleProvider.AddBundleBalance(context.Background(), &bundle.AddBundleBalanceReq{
			UserId:                   int32(userID),
			AccountConsumptionNumber: -1,
		})
		if err != nil {
			service.Error(ctx, err)
			//FIXME 进行回滚
			return
		}
	}
	service.Success(ctx, nil)
	return
}

func BindManager(ctx *gin.Context) {
	var req *cast.BindManagerReq
	var err error
	if err = ctx.ShouldBind(&req); err != nil {
		service.Error(ctx, err)
		return
	}
	if err = req.Validate(); err != nil {
		service.Error(ctx, err)
		return
	}
	_, err = service.CastProvider.BindManager(ctx, req)
	if err != nil {
		service.Error(ctx, err)
		return
	}
	service.Success(ctx, nil)
	return
}

func UpdateMediaAccount(ctx *gin.Context) {
	var req *cast.UpdateMediaAccountReq
	var infoResp *accountFiee.UserInfoResponse
	var err error
	if err = ctx.ShouldBind(&req); err != nil {
		service.Error(ctx, err)
		return
	}
	if err = req.Validate(); err != nil {
		service.Error(ctx, err)
		return
	}
	// 查询艺人的信息
	// 字符串转整型
	artistID, err := strconv.ParseUint(req.ArtistUuid, 10, 64)
	if config.AppConfig.System.AppMode != "dev" {
		infoResp, err = service.AccountFieeProvider.Info(context.Background(), &accountFiee.InfoRequest{
			ID:     artistID,
			Domain: "app",
		})
		if err != nil {
			service.Error(ctx, err)
			return
		}
	} else {
		infoResp = &accountFiee.UserInfoResponse{
			Name:   "小波",
			TelNum: "18288888888",
		}
	}
	req.ArtistName = infoResp.Name
	req.ArtistPhone = infoResp.TelNum
	if _, ok := cast.PlatformIDENUM_name[int32(req.PlatformID)]; !ok {
		service.Error(ctx, errors.New(e.GetMsg(e.InvalidParams)))
		return
	}
	userID, _ := strconv.ParseInt(req.ArtistUuid, 10, 64)
	if req.MediaAccountUuid == "" {
		if err = CheckUserBundleBalance(int32(userID), modelCast.BalanceTypeAccountValue); err != nil {
			service.Error(ctx, err)
			return
		}
		_, err = service.BundleProvider.AddBundleBalance(context.Background(), &bundle.AddBundleBalanceReq{
			UserId:                   int32(userID),
			AccountConsumptionNumber: 1,
		})
		if err != nil {
			service.Error(ctx, err)
			return
		}
	}
	resp, err := service.CastProvider.UpdateMediaAccount(ctx, req)
	if err != nil {
		service.Error(ctx, err)
		if req.MediaAccountUuid == "" {
			_, err = service.BundleProvider.AddBundleBalance(context.Background(), &bundle.AddBundleBalanceReq{
				UserId:                   int32(userID),
				AccountConsumptionNumber: -1,
			})
			if err != nil {
				service.Error(ctx, err)
				return
			}
		}
		return
	}
	service.Success(ctx, resp)
	return
}

func OAuthUrl(ctx *gin.Context) {
	var req *cast.OAuthYoutubeUrlReq
	var resp *cast.OAuthYoutubeUrlResp
	var err error
	if err = ctx.ShouldBind(&req); err != nil {
		service.Error(ctx, err)
		return
	}
	if err = req.Validate(); err != nil {
		service.Error(ctx, err)
		return
	}
	if resp, err = service.CastProvider.OAuthYoutubeUrl(ctx, req); err != nil {
		service.Error(ctx, err)
		return
	}
	service.Success(ctx, resp)
	return
}

func OAuth2Callback(ctx *gin.Context) {
	code := ctx.Query("code")
	state := ctx.Query("state")
	//scope := ctx.Query("scope")
	resp, err := service.CastProvider.OAuthYoutubeToken(ctx, &cast.OAuthYoutubeTokenReq{
		MediaAccountUuid: state,
		Code:             code,
		//scope:             scope,
	})
	if err != nil {
		service.Error(ctx, err)
		return
	}
	service.Success(ctx, resp)
	return
}
